/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.Commons;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.FocusEvent;
import com.google.gwt.event.dom.client.FocusHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;

import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;


/**
 * This class is used to instantiate panels used in title bar. Using FocusPanel allow us to handle events.
 * 
 * @author Jean-Yves Goubet
 *
 */
public class TitlePanel extends Composite {
  
  /**
   * column index : the column index that define the column for the sort function.
   * indexFocusPanel : a focus panel that contains the different objects of the component.
   * 
   * @author Mathieu Crassin
   * 
   */
  // it's the index that defines the column for the sort method
  private int columnIndex;
  
  // it's the focus panel that contains the different objects of the component
  private final IndexFocusPanel indexFocusPanel;
  
  
  /**
   * Constructor of the TitlePanel 
   * 
   * @author Jean-Yves Goubet
   * @param text : text of the label
   * @param columnIndex : the index of the column
   * @param grid : grid in which the component is added
   * @param activeSort : a boolean to define if the component must contain sort information.
   * 
   */
  public TitlePanel(String text, final int columnIndex, final SpecialGrid grid, boolean activeSort) {
    
    // instantiate the panel
    this.columnIndex = columnIndex;
    indexFocusPanel = new IndexFocusPanel(columnIndex, true);
    HorizontalPanel horizontalPanel = new HorizontalPanel();
    TitleLabel titleLabel = new TitleLabel(text, true, columnIndex, false);
    titleLabel.setStyleName("titleStyle");
    titleLabel.setHeight("100%");
    horizontalPanel.add(titleLabel);
    
    // instantiate the order icon
    IndexImage sortOrderIcon = null;
    if (activeSort) {
      sortOrderIcon = new IndexImage(ImageResources.INSTANCE.sortOrderIconTrue(),columnIndex,true);
      horizontalPanel.add(sortOrderIcon);
      horizontalPanel.getWidget(1).setVisible(false);
      horizontalPanel.setCellWidth(sortOrderIcon, "10px");
    }
    
    // add horizontal panel to focus panel
    indexFocusPanel.add(horizontalPanel);
    initWidget(indexFocusPanel);
    
    // modif on style
    horizontalPanel.setWidth("100%");
    horizontalPanel.setCellHorizontalAlignment(titleLabel, HasHorizontalAlignment.ALIGN_CENTER);
    
    // if sort is active, modify style to align icon
    if (activeSort) {
      horizontalPanel.setCellVerticalAlignment(sortOrderIcon, HasVerticalAlignment.ALIGN_MIDDLE);
    }
    
    // check if this is not the actions column
    if (text.compareTo("Actions") != 0) {
      // if yes and sort is active
      if (activeSort) {
        // add click handlers to sort
        indexFocusPanel.addClickHandler(new ClickHandler() {
          public void onClick(ClickEvent arg0) {
            if (grid.getColumnIndex() == columnIndex) {
              grid.setOrder(!grid.getOrder());
            } else {
              grid.setOrder(true);
              grid.setColumnIndex(columnIndex);
            }
            grid.sortAndDisplayList();
          } 
        });
      }
      // add mouse over handler
      indexFocusPanel.addMouseOverHandler(new MouseOverHandler() {
        public void onMouseOver(MouseOverEvent cursorPosition) {
          grid.getCellFormatter().setStyleName(0, columnIndex, "titleOverStyle");
          MouseOutHandlerWithHandlerRegistration mouseOutHandlerWithHandlerRegistration = 
            new MouseOutHandlerWithHandlerRegistration(indexFocusPanel, grid);
          mouseOutHandlerWithHandlerRegistration.setHandlerRegistration(indexFocusPanel.
              addMouseOutHandler(mouseOutHandlerWithHandlerRegistration));
        }
      });
      // add focus handler
      indexFocusPanel.addFocusHandler(new FocusHandler() {
        public void onFocus(FocusEvent arg0) {
          indexFocusPanel.setFocus(false);
        }
      });
    }
  }
  
  
  /**
   * This method is used to display the sort icon depending on the order.
   * 
   * @author Jean-Yves Goubet
   * @param order : a boolean which represents the order for the sort.
   * 
   */
  protected void sortIconDisplay(boolean order) {
    ((IndexImage) ((HorizontalPanel) indexFocusPanel.getWidget()).getWidget(1)).setVisible(true);
    if (order) {
      ((IndexImage) ((HorizontalPanel) indexFocusPanel.getWidget()).getWidget(1)).
      setResource(ImageResources.INSTANCE.sortOrderIconTrue());
    } else {
      ((IndexImage) ((HorizontalPanel) indexFocusPanel.getWidget()).getWidget(1)).
      setResource(ImageResources.INSTANCE.sortOrderIconFalse());
    }
  }
  
  
  /**
   * This method is used to hide the sort icon.
   * 
   * @author Jean-Yves Goubet
   *
   */
  protected void sortIconHide() {
    ((IndexImage) ((HorizontalPanel) indexFocusPanel.getWidget()).getWidget(1)).setVisible(false);
  }
  
  
  /**
   * Getter of the columnIndex parameter
   * 
   * @author Jean-Yves Goubet
   * @return int : the columnIndex value
   * 
   */
  protected int getColumnIndex() {
    return columnIndex;
  }
  
  
  /**
   * Getter of the indexFocusPanel parameter
   * 
   * @author Jean-Yves Goubet
   * @return IndexFocusPanel : the focus panel
   * 
   */
  public IndexFocusPanel getIndexFocusPanel() {
    return indexFocusPanel;
  }
}
